@@ -115,9 +115,9 @@ setMethod("knobs", c("mrgmod", "missing"),  function(x,...) {
   outn <- out@outnames
   
   out <- out %>% as.data.frame
-  out <- dplyr::select_(out,.dots=setdiff(names(out),whatkn))
+  out <- out[,unique(setdiff(names(out),whatkn)),drop=FALSE]
   
-  data <- data %>% dplyr::select_(.dots=c("ID",whatkn))
+  data <- data[,unique(c("ID",whatkn)),drop=FALSE]
   out <- dplyr::left_join(out,data, by="ID") 
   
   new("batch_mrgsims",
