@@ -31,6 +31,8 @@ typedef std::vector<rec_ptr> reclist;
 
 void add_mtime(reclist& thisi, dvec& b, dvec& c, bool debug);
 
+#define NEWREC boost::make_shared<datarecord>
+
 class datarecord {
   
 public:
@@ -62,6 +64,7 @@ public:
   void pos(int pos_) {Pos=pos_;}
   
   short int cmt(){return Cmt;}
+  short int cmtn(){return std::abs(Cmt)-1;}
   
   void output(bool in){Output=in;}
   bool output(){return Output;}
@@ -85,17 +88,19 @@ public:
   void ii(double ii_){Ii = ii_;}
   double ii(){return Ii;}
   
-  void fn(double value){Fn = value;}
-  double fn(){return Fn;}
-  
-  void schedule(std::vector<rec_ptr>& thisi, double maxtime, bool put_ev_first);
+  void schedule(std::vector<rec_ptr>& thisi, double maxtime, bool put_ev_first, double Fn);
   void implement(odeproblem* prob);
   void steady_infusion(odeproblem* prob);
   void steady_bolus(odeproblem* prob);
   
   bool infusion(){return (Evid==1) && (Rate > 0);}
-  bool is_event() {return Evid != 0 ;}
-  bool needs_sorting(){return ((Addl > 0) || (Rate > 0) || (Ss == 1));}
+  bool int_infusion(){return (Evid==1) && (Rate > 0) && (Amt > 0);}
+  bool ss_int_infusion(){return (Evid==1) && (Rate > 0) && (Amt > 0) && (Ss > 0);}
+  bool const_infusion(){return (Evid==1) && (Rate > 0) && (Amt == 0);}
+  bool is_event() {return (Evid != 0) && (Evid != 2) ;}
+  bool is_dose(){return Evid==1;}
+  bool is_event_data() {return (Evid != 0) && (Evid != 2) && Fromdata;}
+  bool needs_sorting(){return ((Addl > 0) || (Ss == 1));}
   
   bool unarmed() {return !Armed;}
   void arm() {Armed=true;}
