@@ -45,7 +45,7 @@ dataobject::dataobject(Rcpp::NumericMatrix _data,
   Data_names = Rcpp::as<Rcpp::CharacterVector>(dimnames[1]);
   
   Idcol = find_position("ID", Data_names);
-  if(Idcol < 0) Rcpp::stop("Could not find ID column in data set.");
+  if(Idcol < 0) throw Rcpp::exception("Could not find ID column in data set.",false);
   
   // Connect Names in the data set with positions in the parameter list
   from_to(Data_names,parnames, par_from, par_to);
@@ -66,7 +66,7 @@ dataobject::dataobject(Rcpp::NumericMatrix _data,
   
   Idcol = find_position("ID", Data_names);
   
-  if(Idcol < 0) Rcpp::stop("Could not find ID column in data set.");
+  if(Idcol < 0) throw Rcpp::exception("Could not find ID column in data set.",false);
   
   for(Rcpp::CharacterVector::iterator it = cmtnames.begin(); it != cmtnames.end(); ++it) {
     *it += "_0";
@@ -135,7 +135,7 @@ void dataobject::locate_tran() {
   
   if(tcol > zeros) {
     tcol = std::find(bg,ed,"TIME") - bg;
-    if(tcol > zeros) Rcpp::stop("Could not find time or TIME column in the data set.");
+    if(tcol > zeros) throw Rcpp::exception("Could not find time or TIME column in the data set.",false);
     lc = false;
   }
   
@@ -168,7 +168,7 @@ void dataobject::locate_tran() {
   if(col[_COL_evid_] > zeros) col[_COL_evid_] = zeros;
   
   if(col[_COL_cmt_] > zeros  && zeros > 0) {
-    Rcpp::stop("Couldn't locate cmt or CMT in data set.");
+    throw Rcpp::exception("Couldn't locate cmt or CMT in data set.",false);
   }
 }
 
@@ -194,7 +194,6 @@ void dataobject::copy_inits(int this_row, odeproblem *prob) {
   }
 }
 
-
 void dataobject::reload_parameters(const Rcpp::NumericVector& PARAM, 
                                    odeproblem *prob) {
   size_t n = par_to.size();
@@ -203,13 +202,11 @@ void dataobject::reload_parameters(const Rcpp::NumericVector& PARAM,
   }
 }
 
-
-
 void dataobject::get_records(recstack& a, int NID, int neq,
                              unsigned int& obscount, unsigned int& evcount,
                              bool obsonly, bool debug) {
   
-  // only look here for events or observations if there is more than one column:
+  // only look here for events or observation if there is more than one column:
   // size_t h=0; warnings
   int j=0;
   int this_cmt;
@@ -228,7 +225,7 @@ void dataobject::get_records(recstack& a, int NID, int neq,
     for(j = this->start(h); j <= this->end(h); ++j) {
       
       if(Data(j,col[_COL_time_]) < lastime) {
-        Rcpp::stop("Problem with time: data set is not sorted by time or time is negative.");
+        throw Rcpp::exception("data set is not sorted by time or time is negative.",false);
       }
       
       lastime = Data(j,col[_COL_time_]);
@@ -239,7 +236,7 @@ void dataobject::get_records(recstack& a, int NID, int neq,
       if(Data(j,col[_COL_evid_])==0) {
         
         if((this_cmt < 0) || (this_cmt > neq)) {
-          Rcpp::stop("cmt number in observation record out of range.");
+          throw Rcpp::exception("cmt number in observation record out of range.",false);
         }
         
         rec_ptr obs = boost::make_shared<datarecord>(
@@ -256,7 +253,7 @@ void dataobject::get_records(recstack& a, int NID, int neq,
       
       // Check that cmt is valid:
       if((this_cmt==0) || (abs(this_cmt) > neq)) {
-        Rcpp::stop("cmt number in dosing record out of range.");
+        throw Rcpp::exception("cmt number in dosing record out of range.",false);
       }
       
       ++evcount;
@@ -272,43 +269,44 @@ void dataobject::get_records(recstack& a, int NID, int neq,
       );
       
       if((ev->rate() < 0) && (ev->rate() != -2) && (ev->rate() != -1)) {
-        Rcpp::stop("Non-zero rate must be positive or equal to -1 or -2");
+        throw Rcpp::exception("Non-zero rate must be positive or equal to -1 or -2",
+                              false);
       }
       
       if((ev->rate() != 0) && (ev->amt() <= 0) && (ev->evid()==1)) {
-        Rcpp::stop("Non-zero rate requires positive amt.");
+        throw Rcpp::exception("Non-zero rate requires positive amt.",
+                              false);
       }
       
       ev->from_data(true);
       if(!obsonly) ev->output(true);
       
+      
       ev->ss(Data(j,col[_COL_ss_]));
       ev->addl(Data(j,col[_COL_addl_]));
       ev->ii(Data(j,col[_COL_ii_]));
       
       if(ev->ii() <= 0) {
         if(ev->addl() > 0) {
-          Rcpp::stop("Found dosing record with addl > 0 and ii <= 0.");
+          throw Rcpp::exception("Found dosing record with addl > 0 and ii <= 0.",
+                                false);
         }
         if(ev->ss()) {
-          Rcpp::stop("Found dosing record with ss==1 and ii <= 0.");
+          throw Rcpp::exception("Found dosing record with ss==1 and ii <= 0.",
+                                false);
         }
       }
-      
       a[h].push_back(ev);
-      
     }
   }
 }
 
-
 void dataobject::get_ids(uidtype* ids) {
   for(int i = 0; i < Data.nrow(); ++i) {
     ids->push_back(Data(i,Idcol)); 
   }
 }
 
-
 void dataobject::check_idcol(dataobject& idat) {
   
   if(idat.ncol() == 0) {return;}
@@ -323,7 +321,7 @@ void dataobject::check_idcol(dataobject& idat) {
   sort_unique(uidata);
   
   if(!std::includes(uidata.begin(),uidata.end(),uthis.begin(),uthis.end())) {
-    Rcpp::stop("ID found in the data set, but not in idata.");
+    throw Rcpp::exception("ID found in the data set, but not in idata.",false);
   }
 }
 
